/*
  EPSI preview to PGM converter

  Boris Tobotras <boris@xtalk.msk.su>
  Copyright (c) 1998

  Distributed under GNU General Public License
*/

# include <assert.h>
# include <ctype.h>
# include <pgm.h>
# include <stdio.h>
# include <unistd.h>

void EPSIerror( void );
int readhexbyte( FILE * );

main( int argc, char *argv[] )
{
	FILE *in = stdin;
	char buf[ 1024 ];			/* Adobe says 255 ought to be
								   enough */ 
	int width, height, depth;
	gray **data;
	
	if ( argc != 1 ) {
		in = fopen( argv[ 1 ], "r" );
		if ( in == NULL ) {
			perror( argv[ 1 ] );
			exit( 1 );
		}
	}
	
	while ( 1 ) {
		if ( fgets( buf, 1024, in ) == NULL
			 && feof( in )
			 && !strncmp( buf, "%%BeginProlog", 13 ) )
			EPSIerror();

		if ( !strncmp( buf, "%%BeginPreview", 14 ) )
			break;
	}
	
	if ( sscanf( buf, "%%%%BeginPreview: %d %d %d", &width, &height,
				 &depth ) != 3 )
		EPSIerror();

	ppm_init( &argc, argv );
	data = pgm_allocarray( width, height );
	assert( data );
	{
		int i, j, accumulated = 0, accumulator = 0;
		for ( i = 0; i < height; ++i ) {
			for ( j = 0; j < width; ) {
				int Byte = readhexbyte( in );
				int bit;
				int bitsRest = 8;
				if ( j + 8 > width )
					bitsRest = width - j;
				for ( bit = bitsRest; bit; --bit ) {
					accumulator <<= 1;
					if ( Byte & ( 1 << bit ) )
						accumulator |= 1;
					if ( ++accumulated == depth ) {
						data[ i ][ j++ ] = accumulator * PGM_MAXMAXVAL
							/ ( 1 << depth ); 
						accumulated = accumulator = 0;
					}
				}
			}
		}
	}
	pgm_writepgm( stdout, data, width, height, PGM_MAXMAXVAL, 0 );
}

void EPSIerror( void ) 
{
	fprintf( stderr, "Illegal EPSI file\n" );
	exit( 1 );
}

int readhexchar( FILE *in )
{
	while ( 1 ) {
		int c = fgetc( in );
		if ( c == EOF )
			EPSIerror();
		if ( c >= '0' && c <= '9' )
			return c - '0';
		if ( c >= 'a' && c <= 'f' )
			return c - 'a' + 10;
		if ( c >= 'A' && c <= 'F' )
			return c - 'A' + 10;
	}
}

int readhexbyte( FILE *in )
{
	return ( readhexchar( in ) << 4 ) | readhexchar( in );
}

